#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""A module for converting integer numbers to/from Esperanto strings.

Examples (Python 2):

    >>> from eonums import int2eo, eo2int, validate_eo
    >>>
    >>> int2eo(22334455)
    u'dudek du milionoj tricent tridek kvar mil kvarcent kvindek kvin'
    >>>
    >>> eo2int(u"cent dudek tri")
    123
    >>> validate_eo(u"dudek cent tri")
    False

Examples (Python 3):

    >>> from eonums import int2eo, eo2int, validate_eo
    >>>
    >>> int2eo(22334455)
    'dudek du milionoj tricent tridek kvar mil kvarcent kvindek kvin'
    >>>
    >>> eo2int("cent dudek tri")
    123
    >>> validate_eo("dudek cent tri")
    False
"""

__version__ = "0.9.1"
__license__ = "GNU General Public Licence v3 (GPLv3)"
__author__ = "Dinu Gherman"
__date__ = "2012-06-20"


import re
import unittest


# constants

# single digits
DIGITS = "nul unu du tri kvar kvin ses sep ok naŭ".split()

# powers of ten
DIGITS10 = DIGITS + ["dek"]
POW10_SMALL = "unu dek cent".split()
POW10_BIG = {3: "mil", 6: "miliono", 9: "miliardo"}
for i in range(12, 63, 6):
    POW10_BIG[i] = DIGITS10[i // 6] + "iliono"
    POW10_BIG[i + 3] = DIGITS10[i // 6] + "iliardo"
POW10_BIG_INV = dict([(v, k) for (k, v) in list(POW10_BIG.items())])
del DIGITS10


# helpers

def digits(num):
    "Convert an integer into a list of decimal digits, e.g. 234 -> [2, 3, 4]."

    return [int(s) for s in str(num)]


def validate_eo(numString, verbose=False):
    """Validate an Esperanto string as a valid integer number.

    Return True or False, e.g.
        "du mil tri milionoj unu" -> False,
        "tri milionoj du mil unu" -> True
        "tridek ducent" -> False
    """

    # check if "big" powers of ten, i.e. 10**(3*k) (k=1,2,3,...), appear
    # with decreasing values of k, excluding e.g. "du mil tri milionoj"
    bigPowers = re.findall("(\w+lionoj?|\w+iliardoj?|mil)", numString)
    for i, p in enumerate(bigPowers):
        if p.endswith("oj"):
            bigPowers[i] = p[:-1]
    bpows = [POW10_BIG_INV[p] for p in bigPowers]
    bpows2 = bpows[:]
    bpows2.sort(reverse=True)
    if bpows != bpows2:
        if verbose:
            print("failing", bpows, bpows2)
        return False

    # check if all occurences of these "big" powers appear
    # exactly once, excluding e.g. "du mil tri mil"
    counts = [bpows.count(p) for p in bpows]
    if counts != [1] * len(bpows):
        if verbose:
            print("failing", counts, [1] * len(bpows))
        return False

    # check if all "small" powers of ten, i.e. 10**k (k=0,1,2),
    # are "well-formed", excluding e.g. "tridek ducent" and "unu du"
    hundreds = re.split("\w+lionoj?|\w+iliardoj?|mil", numString)
    hundreds = [h.strip() for h in hundreds if h.strip()]
    if verbose:
        print("hundreds", hundreds)
    dic = {"digs": "|".join(DIGITS)}
    for h in hundreds:
        if verbose:
            print("testing", repr(h))
        nc = numComponents = len(h.split())
        if nc == 1:
            pat = "^(%(digs)s)?cent$|^(%(digs)s)?dek$|^(%(digs)s)$" % dic
            if not re.match(pat, h):
                if verbose:
                    print("failing (1)", h)
                return False
        elif nc == 2:
            pat1 = "^(%(digs)s)?cent +(%(digs)s)?dek$" % dic
            pat2 = "^(%(digs)s)?cent +(%(digs)s)$" % dic
            pat3 = "^(%(digs)s)?dek +(%(digs)s)$" % dic
            if not re.match("|".join([pat1, pat2, pat3]), h):
                if verbose:
                    print("failing (2)", h)
                return False
        elif nc == 3:
            pat = "^(%(digs)s)?cent +(%(digs)s)?dek +(%(digs)s)$" % dic
            if not re.match(pat, h):
                if verbose:
                    print("failing (3)", h)
                return False
        elif nc == 0 or nc > 3:
            if verbose:
                print("failing (0)", h)
            return False

    return True


# conversion functions

def int2eo(num, validate=True, verbose=False):
    """Convert an (almost) arbitrarily long integer into an Esperanto string.

    Raises a ValueError for numbers >= 10**66.

    Examples:
         234 -> u"ducent tridek kvar"
        1024 -> u"mil dudek kvar"
        2048 -> u"du mil kvardek ok"
    """

    msg = "Unknown Esperanto name for this big number: %d. " % num
    msg += "The largest integer value is 10**66 - 1."
    if num >= 10 ** 66:
        raise ValueError(msg)

    if verbose:
        print("Input:", num)

    if num < 10:
        return DIGITS[num]

    ndigits = digits(num)
    ndigits = ndigits[::-1]
    res = ""
    for (i, n) in enumerate(ndigits):
        nextMax3Digits = ndigits[i:i + 3]
        l = len(nextMax3Digits)
        if verbose:
            print("next3", nextMax3Digits, [1, 0, 0][:l])
        j = i % 3
        eodig = DIGITS[n]
        if n == 1:
            if j in (1, 2) or (i == 3 and nextMax3Digits == [1, 0, 0][:l]):
                eodig = ""
        val = POW10_SMALL[j]
        if j == 0:
            val = ""
            # handle value 10**3
            if i == 3:
                if nextMax3Digits != [0] * l:
                    res = "mil " + res.strip()
            # handle values 10**k, k=6,9,12,...
            elif i > 3:
                # get name for 10**k, e.g. "miliono"
                v = ""
                if nextMax3Digits > [0, 0, 0][:l]:
                    v = POW10_BIG[i]
                # check if plural "j" is needed for "miliono", etc.
                if nextMax3Digits[::-1] > [1, 0, 0][:l][::-1]:
                    v += "j"
                res = v + " " + res.strip()
        if n != 0:
            res = "%s%s " % (eodig, val) + res.strip()

        res = res.strip()

        if verbose:
            print("i:%d, j:%d, n:%d|%s, res:%s" % (i, j, n, eodig, repr(res)))

    if verbose:
        print("Output:", res)
        print()

    # catch expressions which are not "well formed"
    if validate and validate_eo(res) == False:
        raise ValueError("Invalid expression: '%s'" % res)

    return res


def eo2int(numString, validate=True, verbose=False):
    """Convert an integer string in Esperanto into an integer.

    Raises a ValueError when encountering substrings that do not
    describe valid Esperanto numbers, like "du mil tri mil" or "foo".

    Examples:
        u"ducent tridek kvar" -> 234
        u"mil dudek kvar"     -> 1024
        u"du mil kvardek ok"  -> 2048
        u"du what kvardek ok"  -> ValueError
    """

    if verbose:
        print("Input:", numString)

    # catch expressions which are not "well formed"
    if validate and validate_eo(numString) == False:
        raise ValueError("Invalid expression: '%s'" % numString)

    # if we have a single digit number like u"kvin",
    # return its integer value immediately
    if numString in DIGITS:
        return DIGITS.index(numString)

    # split combined expressions like u"ducent" into u"du cent"
    expanded = re.sub("(%s)(cent|dek)" % "|".join(DIGITS[2:]),
        lambda m: m.groups()[0] + " " + m.groups()[1], numString)

    # build a list of eo strings like [u"du", u"cent", u"tri"]
    strList = expanded.split()

    total = 0
    value999 = 0
    value = 1
    for s in strList:
        if verbose:
            format = "s: %s, val: %d, val999: %d, total: %d"
            print(format % (s, value, value999, total))
        if s in DIGITS:
            value = DIGITS.index(s)
        elif s == "dek":
            if value == 0:
                value = 1
            value999 = value999 + value * 10
            value = 0
        elif s == "cent":
            if value == 0:
                value = 1
            value999 = value999 + value * 100
            value = 0
        elif s == "mil":
            value999 = value999 + value
            value999 = max(1, value999)
            total = total + value999 * 1000
            value999 = 0
            value = 0
        elif s in POW10_BIG_INV:
            value999 = value999 + value
            total = total + value999 * (10 ** POW10_BIG_INV[s])
            value999 = 0
            value = 0
        elif s.endswith("oj") and s[:-1] in POW10_BIG_INV:
            value999 = value999 + value
            total = total + value999 * (10 ** POW10_BIG_INV[s[:-1]])
            value999 = 0
            value = 0
        else:
            # encountered sth. that has nothing to do with an Esperanto number
            raise ValueError("Unknown subexpression '%s'" % s)

        if verbose:
            format = "s: %s, val: %d, val999: %d, total: %d"
            print(format % (s, value, value999, total))
            print()

    value999 = value999 + value
    total = total + value999

    if verbose:
        format = "s: %s, val: %d, val999: %d, total: %d"
        print(format % (s, value, value999, total))

    return total
