#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""A module for testing functions in eonums.py.
"""

__version__ = "0.9.1"
__license__ = "GNU General Public Licence v3 (GPLv3)"
__author__ = "Dinu Gherman"
__date__ = "20012-06-20"


import unittest


from eonums import DIGITS, eo2int, int2eo, validate_eo


# tests

TEST_CASES = {}
TEST_CASES.update({
    "digits":
        list(zip(list(range(10)), DIGITS))
})
TEST_CASES.update({
    "doubledigits": [
        (11, "dek unu"),
        (19, "dek naŭ"),
        (20, "dudek"),
        (22, "dudek du"),
        (88, "okdek ok"),
    ],
    "hundreds": [
        (100, "cent"),
        (102, "cent du"),
        (120, "cent dudek"),
        (123, "cent dudek tri"),
        (888, "okcent okdek ok"),
    ],
    "milions": [
        (  1000000, "unu miliono"),
        (  1001000, "unu miliono mil"),
        (  2000000, "du milionoj"),
        ( 10000000, "dek milionoj"),
        ( 21000000, "dudek unu milionoj"),
        (100000000, "cent milionoj"),
    ],
    "big": [
        (      10 ** 63, "unu dekiliardo"),
        (  2 * 10 ** 63, "du dekiliardoj"),
        ( 10 * 10 ** 63, "dek dekiliardoj"),
        (100 * 10 ** 63, "cent dekiliardoj"),
    ],
    "other": [
        (    1000, "mil"),
        (   10000, "dek mil"),
        (  100000, "cent mil"),
        (    1234, "mil ducent tridek kvar"),
        (   11234, "dek unu mil ducent tridek kvar"),
        (    2048, "du mil kvardek ok"),
        (22334455,
             "dudek du milionoj tricent tridek kvar mil kvarcent kvindek kvin"), 
    ],
})


class EsperantoValidationTestCase(unittest.TestCase):
    "Test validation of Esperanto integer numbers."

    def test0(self):
        "Test validation of Esperanto integer numbers"

        validations = [
            ("du triilionoj kvar mil kvin milionoj", False),
            ("du triilionoj kvin milionoj kvar mil", True),
            ("du mil tri milionoj", False),
            ("du mil tri mil", False),
            ("unu miliono mil", True),
            ("du milionoj mil", True),
            ("du mil tri cent", False),
            ("du mil tricent", True),
            ("unu", True),
            ("nulcent nuldek nul", True),
        ]
        for numString, exp in validations:
            res = validate_eo(numString, verbose=False)
            args = (numString, res, exp)
            msg = "Input: '%s', response: %s, expected: %s" % args
            self.assertEqual(res, exp, msg)


class Int2EsperantoTestCase(unittest.TestCase):
    "Test conversion from integers to Esperanto."

    def test_int2eo_digits(self):
        "Convert single digit integers to Esperanto."

        eoMap = TEST_CASES["digits"]
        for i, exp in eoMap:
            res = int2eo(i, verbose=False)
            self.assertEqual(res, exp)

    def test_int2eo_doubledigits(self):
        "Convert selected two digit integers to Esperanto."

        eoMap = TEST_CASES["doubledigits"]
        for i, exp in eoMap:
            res = int2eo(i, verbose=False)
            self.assertEqual(res, exp)

    def test_int2eo_hundreds(self):
        "Convert selected three digit integers to Esperanto."

        eoMap = TEST_CASES["hundreds"]
        for i, exp in eoMap:
            res = int2eo(i, verbose=False)
            self.assertEqual(res, exp)

    def test_int2eo_milions(self):
        "Convert selected integers >= 1e6 to Esperanto."

        eoMap = TEST_CASES["milions"]
        for i, exp in eoMap:
            res = int2eo(i, verbose=False)
            self.assertEqual(res, exp)

    def test_int2eo_big(self):
        "Convert selected really big integers to Esperanto."

        eoMap = TEST_CASES["big"]
        for i, exp in eoMap:
            res = int2eo(i, verbose=False)
            self.assertEqual(res, exp)

    def test_int2eo_other(self):
        "Convert selected other integers to Esperanto."

        eoMap = TEST_CASES["other"]
        for i, exp in eoMap:
            res = int2eo(i, verbose=False)
            self.assertEqual(res, exp)

    def test_int2eo_too_big(self):
        "Convert too big integer to Esperanto."

        self.assertRaises(ValueError, int2eo, 10 ** 70)

    def template_test_failing(self):
        "Test failing case."

        i, exp = 10000000, "dek milionoj"
        res = int2eo(i, verbose=True)
        self.assertEqual(res, exp)


class Esperanto2IntTestCase(unittest.TestCase):
    "Test conversion from Esperanto strings to integers."

    def test_eo2int_digits(self):
        "Convert single digit Esperanto strings to integers."

        eoMap = TEST_CASES["digits"]
        for exp, eo in eoMap:
            res = eo2int(eo, verbose=False)
            self.assertEqual(res, exp)

    def test_eo2int_rest(self):
        "Convert remaining Esperanto strings into integers."

        for kind in TEST_CASES:
            if kind != "digits":
                eoMap = TEST_CASES[kind]
                for exp, eo in eoMap:
                    res = eo2int(eo, verbose=False)
                    self.assertEqual(res, exp)


class RoundtripTestCase(unittest.TestCase):
    "Test conversion from integers to Esperanto and back to integers."

    def test_int2eo2int(self):
        "Convert integers to Esperanto and back to integers."

        for i in range(100000):
            eo = int2eo(i, validate=False, verbose=False)
            j = eo2int(eo, validate=True, verbose=False)
            try:
                self.assertEqual(i, j)
            except AssertionError:
                print("Error: %d -> '%s' -> %d" % (i, eo, j))
                eo = int2eo(i, validate=False, verbose=False)
                j = eo2int(eo, validate=True, verbose=False)
                self.assertEqual(i, j)


if __name__ == "__main__":
    unittest.main()
